// ---------------------------------------------------------------------------
// - Utility.cpp                                                             -
// - standard object library - utility function class implementation         -
// ---------------------------------------------------------------------------
// - This program is free software;  you can redistribute it  and/or  modify -
// - it provided that this copyright notice is kept intact.                  -
// -                                                                         -
// - This program  is  distributed in  the hope  that it will be useful, but -
// - without  any  warranty;  without  even   the   implied    warranty   of -
// - merchantability or fitness for a particular purpose.  In no event shall -
// - the copyright holder be liable for any  direct, indirect, incidental or -
// - special damages arising in any way out of the use of this software.     -
// ---------------------------------------------------------------------------
// - copyright (c) 1999-2006 amaury darsch                                   -
// ---------------------------------------------------------------------------

#include "Utility.hpp"
#include "Exception.hpp"
#include "cgen.hpp"
#include "ccnv.hpp"

namespace afnix {

  // return the maximum long integer value

  long Utility::maxlong (void) {
    // shift index to trick the compiler
    long sidx = 0;
    if (sizeof (long) == 8) sidx = 63;
    if (sizeof (long) == 4) sidx = 31;
    if (sidx == 0) {
      throw Exception ("internal-error", "unknown integer size");
    }
    // compute max value
    return ~(1 << sidx);
  }

  // return the minimum long integer value

  long Utility::minlong (void) {    // shift index to trick the compiler
    long sidx = 0;
    if (sizeof (long) == 8) sidx = 63;
    if (sizeof (long) == 4) sidx = 31;
    if (sidx == 0) {
      throw Exception ("internal-error", "unknown integer size");
    }
    // compute min value
    return (1 << sidx);
  }

  // generate the closest prime to a given integer

  long Utility::toprime (const long value) {
    // the table of prime number
    static const long prime_table [] = {
      37,        73,         157,        307,       601,       1201,
      2417,      4801,       9001,       19001,     37003,     70001,
      130003,    270001,     570001,     1000003,   2000003,   4000037,
      8000017,   20000003,   40000003,   81000001,  160000003, 330000001, 
      600000001, 1000000007, 2000000011, 402500003, 429454979
    };
    static const long prime_size = sizeof (prime_table) / sizeof (long);
    long num = (value < 0) ? -value : value;
    for (long i = 0; i < prime_size; i++) {
      long prime = prime_table[i];
      if (prime > num) return prime;
    }
    return prime_table[prime_size - 1];
  }

  // extract new integer from a string

  t_long Utility::tointeger (const String& value) {
    bool  status     = false;
    const char* data = value.tochar ();
    t_long result = c_atoll (data, status);
    delete [] data;
    if (status == false) 
      throw Exception ("literal-error",
		       "illegal string integer number", value);
    return result;
  }

  // convert an integer to a string

  String Utility::tostring (const long value) {
    char*  buffer = c_ltoa (value);
    String result = buffer;
    delete [] buffer;
    return result;
  }

  // convert an integer to a string

  String Utility::tostring (const t_long value) {
    char*  buffer = c_lltoa (value);
    String result = buffer;
    delete [] buffer;
    return result;
  }

  // return a long random number

  long Utility::longrnd (void) {
    return c_longrnd ();
  }

  // return a long random number

  long Utility::longrnd (const long max) {
    return c_longrnd (max);
  }

  // return a real random number

  t_real Utility::realrnd (void) {
    return c_realrnd ();
  }

  // return a byte random number

  t_byte Utility::byternd (void) {
    return c_byternd ();
  }

  // return a control character from a printable character

  t_quad Utility::toctrl (const char value) {
    return (t_quad) (value - 'a' + 1);
  }
}
